#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "FileList.h"

#include <vector>

using namespace std;

#ifdef _WIN32
#define STRDUP _strdup
#include <io.h>

struct dirent
{
	char *d_name;
};

typedef struct
{
	struct dirent de;
	struct _finddata_t fd;
	intptr_t handle;
} DIR;


DIR *opendir(char *name)
{
	char tmp[256];
	DIR *dir = (DIR*)malloc(sizeof(DIR));
	dir->de.d_name = NULL;
	sprintf(tmp, "%s\\*", name);
	//printf("Looking for %s\n", tmp);
	dir->handle = _findfirst(tmp, &dir->fd);
	return dir;
}

struct dirent *readdir(DIR *dir)
{
	intptr_t rc = dir->handle;
	if(dir->de.d_name)
		rc = _findnext(dir->handle, &dir->fd);
	if(rc == -1)
		return NULL;
	dir->de.d_name = dir->fd.name;
	return &dir->de;
}

void closedir(DIR *dir)
{
	_findclose(dir->handle);
	free(dir);
}
#else
#define STRDUP strdup
#include <dirent.h>
#endif

FileList::FileList(char *dirname, int (*fcb)(const char *)) : filter_cb(fcb)
{
	strcpy(curdir, dirname);
	Enter(dirname);
}

void FileList::Enter(char *dirname)
{
	char tmp[128];
	DIR *dir;
	struct dirent *de;
	int i = 0;
	dir = opendir(dirname);

	if(dir)
	{
		dirty = true;

		for(int i=0; i<fnames.size(); i++)
			free(fnames[i]);

		fnames.clear();

		while((de = readdir(dir)))
		{
			//fprintf(stderr, ">>%s\n", de->d_name);		
			if(de->d_name[0] != '.')
			{
				struct stat ss;
				ss.st_mode = 0;
				sprintf(tmp, "%s%c%s", dirname, SEPARATOR, de->d_name);
				if(stat(tmp, &ss) == 0)
				{
					if(!filter_cb || (ss.st_mode & S_IFDIR) || filter_cb(de->d_name))
					{
						fnames.push_back(STRDUP(tmp));
					}
				}
			}
		}
		closedir(dir);
	}
}



int FileList::Enter(int index)
{
	char old[128] = "";
	if(index == -1)
	{
		char *end = strrchr(curdir, SEPARATOR);
		if(end)
		{
			strcpy(old, curdir);
			*end = 0;
			if(!strchr(curdir, SEPARATOR))
			{
				*end = SEPARATOR;
				end[1] = 0;
			}
		}
	}
	else
		strcpy(curdir, fnames[index]);

	Enter(curdir);

	if(strlen(old))
	{
		printf("Old was '%s'\n", old);
		for(int i=0; i<fnames.size(); i++)
		{
			printf("'%s'\n", fnames[i]);
			if(strcmp(fnames[i], old) == 0)
				return i;
		}
	}

	return 0;

}

FileList::~FileList()
{
	for(int i=0; i<fnames.size(); i++)
		free(fnames[i]);
}


int FileList::Read(int index, char *dest, int offset, int length)
{
	int rc = -1;
	FILE *fp = fopen(fnames[index], "rb");
	if(fp)
	{
		fseek(fp, offset, SEEK_SET);
		rc = fread(dest, 1, length, fp);
		fclose(fp);
	}
	return rc;
}

const char *FileList::GetFileName(int index)
{
	return fnames[index];
}

vector<FileRef> &FileList::GetNames()
{
	if(dirty)
	{
		filerefs.clear();
		for(int i=0; i<fnames.size(); i++)
		{
			struct stat ss;
			ss.st_mode = 0;
			ss.st_size = 0;
			if(stat(fnames[i], &ss) == 0)
			{
				//if((ss.st_mode & S_IFDIR) || (strcmp(p, ext) == 0))
				{
					FileRef fr;

					fr.name = strrchr(fnames[i], SEPARATOR);
					if(!fr.name)
						fr.name = fnames[i];
					else
						fr.name++;

					if(ss.st_mode & S_IFDIR)
						fr.size = -1;
					else
						fr.size = ss.st_size;

					filerefs.push_back(fr);
				}
			}
		}
		dirty = false;
	}

	return filerefs;

}




