#include <SDL.h>

#include <stdio.h>
#include <fcntl.h>
#ifdef _WIN32
#include <io.h>
#endif

#include <vector>

#include "SDLScreen.h"
#include "FileList.h"


#include <pogo/screen.h>
#include <pogo/font.h>
#include <pogo/device.h>
#include <pogo/console.h>

#define LINEDIST 0

static ScreenSettings ss;

void bevel_box(SDL_Surface *surf, int x, int y, int w, int h)
{
	SDL_Rect rect;
	rect.x = x-2;
	rect.y = y-2;
	rect.w = w+4;
	rect.h = h+4;
	SDL_FillRect(surf, &rect, UINT2RGB16(ss.beveldark));
	rect.x = x;
	rect.y = y;
	rect.w = w+2;
	rect.h = h+2;
	SDL_FillRect(surf, &rect, UINT2RGB16(ss.bevellight));
}

void raster(SDL_Surface *surf, int x, int y, int width, int height, int col0, int col1)
{
	int r = col0>>16;
	int g = (col0>>8)&0xff;
	int b = col0&0xff;
	int dr = ((col1>>16) - r) * 256 / width;
	int dg = (((col1>>8) & 0xff) - g) * 256 / width;
	int db = ((col1&0xff) - b) * 256 / width;

	r <<= 8;
	g <<= 8;
	b <<= 8;

//	printf("%d %d %d -> %d %d %d\n", r, g, b, dr, dg, db);

	int w = width;
	unsigned short *pix = (unsigned short *)surf->pixels + x + y*surf->w;
//	printf("%p %d %d\n", pix, x, y*width);
	while(w--)
	{
		unsigned short col = TO_RGB16((r>>8), (g>>8), (b>>8));
//		printf("%04x\n", col);
		int h = height;
		while(h--)
		{
			*pix = col;
			pix += surf->w;
		}
		pix -= (surf->w*height-1);
		r += dr;
		g += dg;
		b += db;
	}
}

Pane::Pane(int x, int y, int w, int h, Screen *scr)
{
	if(!scr)
		scr = &g_screen;

	font = scr->font1;
	screen = &scr->screen;

	con_fd = scr->con_fd;
	for(int i=0; i<4; i++)
		fonts[i] = scr->font0;


	dirty = REDRAW | RESIZE;

	m_x = x;
	m_y = y;
	m_w = w;
	m_h = h;

	Clear(true);
}

int Pane::Text(const char *t, int x, int y)
{
	int len;

	if(y >= 0)
	{
		int l = m_w - (x < 0 ? 0 : x*font->charwidth);
		char tmp[128];
		strcpy(tmp, t);
		char *endp = &tmp[strlen(tmp)-1];
		while((endp > tmp) && (Text(tmp, -1, -1) > l))
			*endp-- = 0;

		len = font_text(font, (char *)tmp, -1, -1);

		if(x < 0)
			font_text(font, (char *)tmp, m_x + m_w - len + x*font->charwidth, m_y + y*(font->height+LINEDIST));
		else
			font_text(font, (char *)tmp, m_x + x*font->charwidth, m_y + y*(font->height+LINEDIST));
	}
	else
		len = font_text(font, (char *)t, -1, -1);

	return len;
}

void Pane::Clear(bool bevel)
{
	if(m_w && m_h)
	{
		if(bevel)
			bevel_box(*screen, m_x, m_y, m_w, m_h);

		SDL_Rect rect;
		rect.x = m_x;
		rect.y = m_y;
		rect.w = m_w;
		rect.h = m_h;
		SDL_FillRect(*screen, &rect, ss.bgcol);
		dirty |= REDRAW;
	}
}

void Pane::Render()
{
	if(dirty && m_w && m_h)
	{
		if(dirty & RESIZE)
			Clear(true);

		dirty = 0;

		SDL_LockSurface(*screen);

		font_setcolor(ss.textcol, ss.bgcol);
		font_settarget((*screen)->pixels, ss.width, ss.height, 0);

		if(Update())
			SDL_UpdateRect(*screen, m_x, m_y, m_w, m_h);
		SDL_UnlockSurface(*screen);
	}
}

bool Pane::Update()
{
	return true;
}

void Pane::Move(int x, int y)
{
	m_x = x;
	m_y = y;
	dirty |= REDRAW;
}

void Pane::Resize(int w, int h)
{
	m_w = w;
	m_h = h;
	dirty |= (REDRAW | RESIZE);
}


using namespace std;

	const unsigned char digits[] =
	{
		1,1,1,
		1,0,1,
		1,0,1,
		1,0,1,
		1,1,1,

		0,0,1,
		0,0,1,
		0,0,1,
		0,0,1,
		0,0,1,

		1,1,1,
		0,0,1,
		1,1,1,
		1,0,0,
		1,1,1,

		1,1,1,
		0,0,1,
		1,1,1,
		0,0,1,
		1,1,1,

		1,0,1,
		1,0,1,
		1,1,1,
		0,0,1,
		0,0,1,

		1,1,1,
		1,0,0,
		1,1,1,
		0,0,1,
		1,1,1,

		1,1,1,
		1,0,0,
		1,1,1,
		1,0,1,
		1,1,1,

		1,1,1,
		0,0,1,
		0,0,1,
		0,0,1,
		0,0,1,

		1,1,1,
		1,0,1,
		1,1,1,
		1,0,1,
		1,1,1,

		1,1,1,
		1,0,1,
		1,1,1,
		0,0,1,
		0,0,1,
	};



bool LedControl::Update()
{
	int width = (*screen)->w;

	unsigned short *pix = (unsigned short *)(*screen)->pixels + m_x + width * m_y;

	unsigned char *data = (unsigned char *)display->pixels;
	int mod = width - display->w;
	int height = display->h;

//	printf("%d %d %d\n", mod, width, height);

	while(height--)
	{
		int w = display->w;
		while(w--)
			*pix++ = clut[*data++];
		pix += mod;
	}

	return true;

}


LedControl::LedControl(char *fname, int x, int y, unsigned int dcol, unsigned int lcol, Screen *scr) :
	Pane(x, y, 0, 0, scr)
{

	dark = UINT2RGB16(dcol);
	light = UINT2RGB16(lcol);

	for(int i = 1; i<256; i++)
		clut[i] = dark;
	clut[0] = 0;

	clut[SPEAKER] = light;
	clut[SOLID] = light;
	clut[SONG] = light;

	display = SDL_LoadBMP(fname);

	Resize(display->w, display->h);
	//Clear();

	//Pane::Pane(x, y, display->w, display->h, scr);

}

void LedControl::SetNumbers(int led, int val, int digs)
{
	for(int n=digs-1; n>=0; n--)
	{
		int d = val % 10;
		val = val/10;
		const unsigned char *ptr = &digits[d * 15];
		//printf("%d\n", d);
		for(int i=0; i<15; i++)
			clut[led + n*15 + i] = (ptr[i] ? light : dark);
	}
}

void LedControl::SetLed(int led, int val)
{
	int i;

	dirty = true;

	switch(led)
	{
	case SPEAKER:
	case SOLID:
	case SONG:
	case REPEAT:
	case SHUFFLE:
	case PLAYING:
		clut[led] = (val ? light : dark);
		break;
	case VOLUME:
		for(i=0; i<10; i++)
			clut[VOLUME + i] = (val > i ? light : dark);
		break;
	case SONGNO:
	case MAXSONG:
		SetNumbers(led, val, 2);
		break;
	case LENGTH:
		SetNumbers(led + 3*15, val % 60, 2);
		SetNumbers(led, val / 60, 3);
		break;
	case BUSY:
		for(i=0; i<6; i++)
			clut[BUSY + i] = (val > i ? light : dark);
		break;
	case BAR0:
	case BAR1:
		for(i=0; i<12; i++)
			clut[led + i] = (val > i ? light : dark);
		break;
	case EQ0:
	case EQ1:
	case EQ2:
	case EQ3:
		for(i=0; i<5; i++)
			clut[led + i] = (val > i ? light : dark);
		break;
	default:
		dirty = false;
	}
}

Screen::Screen() :
con_fd(-1),
screen(NULL)
{
}

PogoDevice native;

int native_open(const char *name, int flags)
{
	return (int)open(name, 0);
}

int native_read(int fd, void *dest, int size)
{
	return read(fd, dest, size);
	//return fread(dest, 1, size, (FILE *)fd);
}

int native_write(int fd, const void *dest, int size)
{
	return -1;
}

void native_close(int fd)
{
	//fclose((FILE *)fd);
	close(fd);
}

int native_lseek(int fd, int offset, int origin)
{
	//return fseek((FILE *)fd, offset, SEEK_SET);
	return lseek(fd, offset, origin);
}

int native_tell(int fd)
{
	return -1;
}

int native_ioctl(int fd, int request, ...)
{
	return -1;
}

int native_remove(const char *name)
{
	return -1;
}

int native_stat(const char *name, struct pogo_stat *buffer)
{
	return -1;
}


ScreenSettings *Screen::GetSettings()
{
	return &settings;
}

void Screen::SetSettings(ScreenSettings *s)
{
	settings = *s;
}

void Screen::Resize(int w, int h)
{
	w = (w+3) & 0xFFFC;

	ss.width = w;
	ss.height = h;
	SDL_FreeSurface(screen);
	screen = SDL_SetVideoMode(w, h, 16, 0);//);

	screen_init();
	console_set_target(screen->pixels, ss.width);
	font_settarget(screen->pixels, ss.width, ss.height, 0);

	SDL_FillRect(screen, NULL, UINT2RGB16(ss.bevelmid));
//	font_text(font1, "OldPlay Test Version - by Sasq", 5, 1);
}

void Screen::Init()
{
	ss = settings;

	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK | SDL_INIT_NOPARACHUTE);
	screen = SDL_SetVideoMode(ss.width, ss.height, 16, 0);//);
	SDL_JoystickOpen(0);
	SDL_ShowCursor(0);

	pogo_device_init();
	memset(&native, 0, sizeof(PogoDevice));

	native.open = native_open;
	native.read = native_read;
	native.close = native_close;
	native.lseek = native_lseek;

	pogo_device_register(&native, "native", NULL, -1);

	font0 = font_load(ss.font0);
	font1 = font_load(ss.font1);

	if(!font0)
		 font0 = font_getdefault();
	if(!font1)
		 font1 = font_getdefault();

	screen_init();
	console_set_target(screen->pixels, ss.width);
	font_settarget(screen->pixels, ss.width, ss.height, 0);

	//screen_set(0, 0, 320, 240, 127);
	SDL_FillRect(screen, NULL, UINT2RGB16(ss.bevelmid));

	//SDL_Surface *bg = SDL_LoadBMP("back.bmp");
	//if(bg)
	//	SDL_BlitSurface(bg, NULL, screen, NULL);

	console_init();
	con_fd = pogo_open("/dev/console", 0);

	//font1->flags |= FFLG_TRANSP;

	//raster(screen, 2, 2, screen->w-4, font1->height + 2, 0xA6BADB, 0xA6BADB);

	font_setcolor(0xFFFFFF, ss.bevelmid);
	int f = font1->flags;
	font1->flags |= FFLG_TRANSP;
	font_setcolor(0x000000, ss.bevelmid);
	font_text(font1, "OldPlay v0.8 - by Sasq", 6, 3);
	font_setcolor(0xFFFFFF, ss.bevelmid);
	font_text(font1, "OldPlay v0.8 - by Sasq", 5, 2);
	font1->flags = f;
	//font1->flags &= ~FFLG_TRANSP;
	
	int mainwin = -1;

	//cprintf("back.bmp = %p\n", bg);


	//pogo_ioctl(con_fd, CC_OPENWIN, 210, 143, 104, 80, fonts, CONWIN_FIXED);
	//pogo_ioctl(con_fd, CC_SETCOLOR, CCOL_BLACK, CCOL_YELLOW);
	//pogo_ioctl(con_fd, CC_CLEAR);
	//pogo_write(con_fd, "RetroBliPx#RetroBliP#RetroBliP#RetroBliP#RetroBliP#RetroBliP#RetroBliP", 50);

	//led_init();
	//fixup_led(led);
	//set_length(12345);
	//set_volume(7);
	//render_led(((unsigned short *)screen->pixels) + 90 * 320 + 212, 320);

}

void Screen::CreateConsole(int x, int y, int w, int h)
{
	Font *fonts[4];

	for(int i=0; i<4; i++)
		fonts[i] = font1;

	mainwin = pogo_ioctl(con_fd, CC_OPENWIN, x, y, w, h, fonts, CONWIN_STREAM);
	bevel_box(screen, x, y, w, h);
	pogo_ioctl(con_fd, CC_SETCOLOR, CCOL_GREEN, CCOL_BLACK);
	pogo_ioctl(con_fd, CC_CLEAR);
}

int Screen::PrintV(char *fmt, va_list vl)
{
	char tmp[256];
	int rc;

	if(con_fd < 0 || mainwin < 0)
		return -1;

	rc = vsprintf(tmp, fmt, vl);
	pogo_ioctl(con_fd, CC_SETWIN, mainwin);
	pogo_write(con_fd, tmp, rc);
	pogo_ioctl(con_fd, CC_FLUSH);
	if(screen)
		SDL_UpdateRect(screen, 0, 0, 0, 0);

	return rc;
}

int Screen::Print(char *fmt, ...)
{
	va_list vl;
	va_start(vl, fmt);
	int rc = PrintV(fmt, vl);
	va_end(vl);
	return rc;
}

Screen g_screen;

int cprintf(char *fmt, ...)
{
	va_list vl;
	va_start(vl, fmt);
	int rc = g_screen.PrintV(fmt, vl);
	va_end(vl);
	return rc;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

ListPane::ListPane(int x, int y, int w, int h, Screen *scr) :
	Pane(x, y, w, h, scr)
{
	start = marked = 0;
	lines = h / (font->height+LINEDIST) - 2;
	dirty = REDRAW;
	delay = 0;
}

void ListPane::Resize(int w, int h)
{
	m_w = w;
	m_h = h;
	dirty |= (REDRAW | RESIZE);
	lines = h / (font->height+LINEDIST) - 2;
}


bool ListPane::Update()
{

	char buf[256];

	//SDL_Rect rect;
	//rect.x = m_x;
	//rect.y = m_y;
	//rect.w = m_w;
	//rect.h = m_h;
	//SDL_FillRect(screen, &rect, ss.bgcol);
	Clear(false);
	dirty = 0;

	vector<FileRef> &finfo = file_list->GetNames();

	int count = (int)finfo.size();

	if(start !=0)
	{
		//pogo_ioctl(con_fd, CC_GOTOXY, 1, 0);
		//pogo_write(con_fd, "..", 2);
		Text("...", 1, 0);
	}
	if(count > start+lines)
	{
		//pogo_ioctl(con_fd, CC_GOTOXY, 1, lines+1);
		//pogo_write(con_fd, "..", 2);
		Text("...", 1, lines+1);

	}

	vector<FileRef>::iterator fi = finfo.begin() + start;

	int l = m_w - Text("9999M  ", -1, -1);

	for(int i = start; i<count && i<(start+lines); i++)
	{
		int bg = 0;
		int fg = 0xFFFFFF;

		if(i == marked)
		{
			bg = 0x000080;
			SDL_Rect r;
			r.x = m_x;
			r.y = m_y + (1+i-start)*(font->height+LINEDIST);
			r.w = m_w;
			r.h = font->height+LINEDIST;
			SDL_FillRect(*screen, &r, UINT2RGB16(bg));
		}

		//	font_setcolor(0xFFFFFF, 0x000000);

		FileRef &fr = *fi;

		char tmp[128];

		strcpy(tmp, fr.name);

		char *endp = &tmp[strlen(tmp)-1];

		while((endp > tmp) && (Text(tmp, -1, -1) > l))
			*endp-- = 0;

		fi++;
		if(fr.size == -1)
		{
			font_setcolor(0xFF0000, bg);
			//fg = 0xFF0000
			sprintf(buf, "[%s]", tmp);
			Text(buf, 1, 1+i-start);
		}
		else
		{
			font_setcolor(fg, bg);
			//sprintf(buf, "%s", fr.name);
			Text(tmp, 1, 1+i-start);
			if(fr.size < 1024)
				sprintf(buf, "%4dB", fr.size);
			else
			if(fr.size < 1024*1024)
				sprintf(buf, "%4dK", fr.size / 1024);
			else
				sprintf(buf, "%4dM", fr.size / (1024*1024));
			Text(buf, -1, 1+i-start);
		}

		//if(i == marked)
		//	font_setcolor(0xFFFF00, 0x000000);
	}

	return true;
}

void ListPane::DoCommand(int cmd)
{
	int oldmarked = marked;

//	last_cmd = cmd;

	switch(cmd)
	{
	case UP:
		marked--;
		break;
	case DOWN:
		marked++;
		break;
	case PGUP:
		marked -= lines;
		break;
	case PGDN:
		marked += lines;
		break;
	}

	int oldstart = start;

	vector<FileRef> &finfo = file_list->GetNames();

	int count = (int)finfo.size();

	if(marked < 0)
		marked = 0;
	if(marked >= count)
		marked = count-1;

	while(marked >= start+lines)
		start += 3;
	while(marked < start)
		start -= 3;

	if(start > count-lines)
		start = count-lines;

	if(start < 0)
		start = 0;

	if((start != oldstart) || (marked != oldmarked))
		dirty = REDRAW;

}








TextDisplay::TextDisplay(int x, int y, int w, int h, Screen *scr) :
	Pane(x, y, w, h, scr)
{
	//conwin = pogo_ioctl(con_fd, CC_OPENWIN, x, y, w, h, fonts, CONWIN_FIXED);
	pogo_ioctl(con_fd, CC_SETCOLOR, CCOL_YELLOW, CCOL_BLACK);
}

void TextDisplay::PutText(int x, int y, const char *str)
{
	//pogo_ioctl(con_fd, CC_SETWIN, conwin);
	//pogo_ioctl(con_fd, CC_GOTOXY, x, y);
	//pogo_write(con_fd, str, strlen(str));
	dirty |= REDRAW;
}

void TextDisplay::Clear(bool bevel)
{
	//pogo_ioctl(con_fd, CC_SETWIN, conwin);
	//pogo_ioctl(con_fd, CC_CLEAR);
	dirty |= REDRAW;
}


SongInfoPane::SongInfoPane(int x, int y, int w, int h, Screen *scr) :
	Pane(x, y, w, h, scr)
{
	strcpy(name, "");
	strcpy(author, "");
	strcpy(copyright, "");
	strcpy(format, "");
	length = -1;
}


bool SongInfoPane::Update()
{
	char tmp[128];
	Clear(false);
	dirty = 0;
	int y = 0;

	int fh = font->height + LINEDIST;

	int flgs = font->flags;
	font->flags |= FFLG_TRANSP;

	if(strlen(name))
	{
		font_setcolor(ss.textcol, 0x438AB3);
		raster(*screen, m_x, m_y+fh*y, m_w, font->height, 0x438AB3, 0x63AAD3);
		Text("Name", 0, y);
		font_setcolor(ss.textcol, ss.bgcol);
		Text(name, 1, y+1);
		y += 2;
	}

	if(strlen(author))
	{
		font_setcolor(ss.textcol, 0x438AB3);
		raster(*screen, m_x, m_y+fh*y, m_w, font->height, 0x438AB3, 0x63AAD3);
		Text("Artist", 0, y);
		font_setcolor(ss.textcol, ss.bgcol);
		Text(author, 1, y+1);
		y += 2;
	}

	if(strlen(copyright))
	{
		font_setcolor(ss.textcol, 0x438AB3);
		raster(*screen, m_x, m_y+fh*y, m_w, font->height, 0x438AB3, 0x63AAD3);
		Text("Copyright", 0, y);
		font_setcolor(ss.textcol, ss.bgcol);
		Text(copyright, 1, y+1);
		y += 2;
	}

	if(strlen(format))
	{
		font_setcolor(ss.textcol, 0x438AB3);
		raster(*screen, m_x, m_y+fh*y, m_w, font->height, 0x438AB3, 0x63AAD3);
		Text("Format", 0, y);
		font_setcolor(ss.textcol, ss.bgcol);
		Text(format, 1, y+1);
		y += 2;
	}

	if(length > 0)
	{
		font_setcolor(ss.textcol, 0x438AB3);
		raster(*screen, m_x, m_y+fh*8, m_w, font->height, 0x438AB3, 0x63AAD3);
		sprintf(tmp, "Length: %02d:%02d", length/60, length % 60);
		Text(tmp, 0, 8);
		y++;
	}

	font->flags = flgs;

	return true;
}

void SongInfoPane::SetArtist(const char *a)
{
	strcpy(author, a);
	dirty |= REDRAW;
}

void SongInfoPane::SetCopyright(const char *c)
{
	strcpy(copyright, c);
	dirty |= REDRAW;
}

void SongInfoPane::SetName(const char *n)
{
	strcpy(name, n);
	dirty |= REDRAW;
}

void SongInfoPane::SetFormat(const char *n)
{
	if(strcmp(format, n) != 0)
	{
		strcpy(format, n);
		dirty |= REDRAW;
	}
}

void SongInfoPane::SetLength(int l)
{
	if(length != l)
	{
		length = l;
		dirty |= REDRAW;
	}
}


