
#include <sidplay/player.h>
#include <sidplay/fformat.h>
#include <sidplay/myendian.h>

#include "plugin.h"

class SidState
{
public:
	SidState(emuEngine *e = NULL, sidTune *t = NULL)
	{
		engine = e;
		tune = t;
		playing = false;
	}

	emuEngine *engine;
	sidTune *tune;
	bool playing;
	int song;
};

static SidState state;

static struct sound_plugin plugin;


static void init_engine()
{
	SidState *s = &state;

	s->playing = false;
		
	if(!s->engine)
	{
		s->engine = new emuEngine();
	
		if(!s->engine->verifyEndianess())
		{
			exit(1);
		}

		// Get the default configuration.
		struct emuConfig myEmuConfig;
		s->engine->getConfig(myEmuConfig);
		myEmuConfig.frequency = 22050;
		myEmuConfig.channels = SIDEMU_STEREO;
		myEmuConfig.bitsPerSample = SIDEMU_16BIT;
		myEmuConfig.sampleFormat = SIDEMU_SIGNED_PCM;
		myEmuConfig.volumeControl = 
		s->engine->setConfig(myEmuConfig);
	}
}

static int init_data(char *name, void *data, int size)
{
	// Initialize the SID-Emulator Engine to defaults.
	init_engine();
	SidState *s = &state;
	if(s->tune)
		delete s->tune;
	s->tune = NULL;

	sidTune *myTune = new sidTune((const ubyte *)data, size);
	if (!myTune)  
		return -1;

	if(!myTune->getStatus())
	{
		delete myTune;
		return -1;
	}

	struct sidTuneInfo mySidInfo;
	myTune->getInfo(mySidInfo);

	plugin.name = mySidInfo.nameString;
	plugin.author = mySidInfo.authorString;
	plugin.subtunes = mySidInfo.songs;
	plugin.tune = mySidInfo.startSong - 1;
	plugin.length = -1;

	s->song = mySidInfo.startSong;
	s->tune = myTune;

	if(!s->tune)
		return -2;

	if(!sidEmuInitializeSong(*(s->engine), *(s->tune), s->song))
		return -3;

	s->engine->resetSecondsThisSong();
	s->playing = true;

	return 0;
}

static int init_file(char *fname)
{
	// Initialize the SID-Emulator Engine to defaults.
	init_engine();
	SidState *s = &state;
	if(fname)
	{
		if(s->tune)
			delete s->tune;
		s->tune = NULL;

		sidTune *myTune = new sidTune(fname);
		if (!myTune)  
			return -1;

		struct sidTuneInfo mySidInfo;
		myTune->getInfo(mySidInfo);

		plugin.name = mySidInfo.nameString;
		plugin.author = mySidInfo.authorString;
		plugin.subtunes = mySidInfo.songs;
		plugin.tune = mySidInfo.startSong - 1;
		plugin.length = -1;

		s->song = mySidInfo.startSong;
		s->tune = myTune;
	}

	if(!s->tune)
		return -2;


	if(!sidEmuInitializeSong(*(s->engine), *(s->tune), s->song))
		return -3;

	s->engine->resetSecondsThisSong();
	s->playing = true;

	return 0;
}

static int fill_buffer(signed short *dest, int len)
{
	if(state.playing)
		sidEmuFillBuffer(*(state.engine), *(state.tune), dest, len);
	else
		memset(dest, 0, len);

	return 0;
}

static int set_position(int seconds, int subtune)
{
	state.song = subtune+1;
	init_file(NULL);
	return 0;
}

static int is_ext(const char *s, const char *ext)
{
	const char *sext = strrchr(s, '.');
	if(!sext)
		sext = "";
	return (strcmp(ext, sext) == 0);
}

static int can_handle(const char *name)
{
	return (is_ext(name, ".sid"));
}

struct __cdecl sound_plugin *init_sound_plugin()
{
	memset(&plugin, 0, sizeof(plugin));

	plugin.freq = 22050;
	plugin.channels = 2;
	plugin.init_file = init_file;
	plugin.init_data = init_data;
	plugin.fill_buffer = fill_buffer;
	plugin.set_position = set_position;
	plugin.can_handle = can_handle;



	return &plugin;
}

