/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <math.h>
#include <string.h>
extern "C" {
#include "fft/CMP_fft.h"
}

#include <SDL.h>

#include "Equalizer.h"


#define FALLOFF 3
#define DIVIDE 16
//#define HEIGHT 50

// Analyze batch and queue for rendering
void Equalizer::WriteSamples(short *samples, int len)
{
	int i,x,y;

	fft(samples, table, len);

	max_hi = (m_h-5);
	dirty = REDRAW;

	for(i=0; i<barcount; i++)
	{
		int start = startpos[i];
		int stop = startpos[i+1];
		int v = 0;

		for(x=start; x<stop; x+=2)
			v += (table[x]+table[x+1]);
		v /= DIVIDE;

		if(v > (m_h-5)) v = (m_h-5);

		if(v > eq[i])
			eq[i] = v;
	}


}


//static int lastn = -1;

bool Equalizer::Update()
{
	uint32 *p, *e;
	int x,y;

	if(max_hi <= 0)
		return false;
	max_hi -= FALLOFF;

	dirty = REDRAW;

	int width = (*screen)->w;

	unsigned short *ptr = (unsigned short *)(*screen)->pixels + m_x + m_y * (*screen)->w;
	unsigned short *scr = ptr;

	p = (uint32*)ptr;
	for(y=0; y<m_h/2; y++)
	{
		for(x=0; x<m_w/2; x++)
			*p++ = 0;
		p += (width-(m_w/2));
	}

	int eh = m_h & 0xFFFE;

	for(x=0; x<m_w; x+= barsize)
	{
		y = eq[x/barsize];
		eq[x/barsize] -= FALLOFF;
		if(eq[x/barsize] < 0)
			eq[x/barsize] = 0;


		p = (uint32*)&scr[x+(eh-2)*width];
		e = (uint32*)&scr[x+(eh-y)*width];
		y = 63;
		for(; p > e; p-=width)
		{
			int bs = (barsize/2)-1;
			uint32 *pp = p;
			while(bs--)
				*pp++ = colors[y];
			*pp = colors2[y];
			y-=2;
		}
	}

	//g_screen.Lock();
	//g_screen.Update();
	return true;
}


Equalizer::Equalizer(int x, int y, int w, int h, Screen *scr) :
	Pane(x, y, w, h, scr)
{
	int r = 0;
	int b = 0;
	int g = 255;
	uint32 l;

	max_hi = 0;
	barcount = 26;
	barsize = w / barcount;

	int numBuckets = 26;
	float startfreq=500;
	float endfreq=2200;
	int from = 0;

	float hzperslot = 22050.0F / (2048.0F/2.0F);

	float curfreq=startfreq;
	float mf=exp(log(endfreq/startfreq)/(numBuckets+1));
	float nextfreq=curfreq*mf;

	int i;

	memset(eq, 0, sizeof(eq));

	for(i=0;i<=numBuckets;i++)
	{
		startpos[i] = from;
		while(curfreq<nextfreq)
		{
				//currentBucket[i] += p[from]*p[from]+p[from+1]*p[from+1];
				curfreq+=hzperslot;
				from+=2;
		}
		nextfreq*=mf;
	}
	startpos[i] = from;


	for(int i=0; i<31; i++)
	{
		l = TO_RGB16(r,g,b);
		colors[i] = l | (l<<16);
		colors2[i] = l;
		r+= 256/32;
	}
		
	for(int i=31; i<64; i++)
	{
		l = TO_RGB16(r,g,b);
		colors[i] = l | (l<<16);
		colors2[i] = l;
		g -= 256/32;
		if(g < 0) g = 0;
	}

	dirty = REDRAW;

}
