/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "PlayList.h"

#include "zipint.h"

#include <vector>

#ifdef _WIN32
#define STRDUP _strdup
#else
#define STRDUP strdup
#endif

using namespace std;

PlayList::PlayList(int (*fcb)(const char *))
: FileList(NULL, fcb)
{
	current = -1;
	changed = 0;
}

PlayList::~PlayList()
{
}

void PlayList::AddFile(const char *name, int size, IFileRef *fr)
{
	PLData data;

	data.name = string(name);
	data.size = size;
	data.fr = fr;
	filerefs.push_back(data);
	changed++;
}

IFileRef *PlayList::GetFileRef(int index)
{
	return new PLFileRef(filerefs[index].fr);
}
void PlayList::RemoveFile(int n)
{

	if(n >= 0 && n < filerefs.size())
	{
		if(n == current)
			current = -1;
		else
		if(n < current)
			current--;
		filerefs.erase(filerefs.begin() + n);
		if(marked >= filerefs.size())
			marked--;
		changed++;
	}

	

}

void PlayList::MoveFile(int from, int to)
{
	if(to == current)
		current++;

	filerefs.insert(filerefs.begin() + to, *(filerefs.begin() + from));

	if(to < current)
		current--;

	if(from > to)
		from++;
	RemoveFile(from);



	changed++;
}
