/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <SDL.h>
//#include "plugin.h"
#include <vector>
#include "FileList.h"
#include "ZipFileList.h"
#include "PlayList.h"
#include "SDLScreen.h"

#include "MusicPlayer.h"
#include "Input.h"
#include "Fifo.h"
#include "Equalizer.h"

#ifndef _WIN32
#define stricmp strcasecmp
#define strnicmp strncasecmp
#include <unistd.h>
#else
#include <direct.h>
#endif

using namespace std;

MusicPlayer *mp = NULL;

static int is_ext(const char *s, const char *ext)
{
	const char *sext = strrchr(s, '.');
	if(!sext)
		sext = "";
	return (stricmp(ext, sext) == 0);
}


int plugin_filter_cb(const char *name)
{
	//printf("Checking %s\n", name);

	if(mp->CanHandle(name))
		return 1;

	if(is_ext(name, ".zip"))
		return 1;

	return 0;
}



ListPane *pane;
SongInfoPane *songInfo;
LedControl *lcd;
Equalizer *eq = NULL;
Screen::Settings settings;
#define ABS(x) ((x) < 0 ? (-x) : (x))

void render_eq()
{
	short buff[4096*2];
	short buff2[2048*2];

	int l = mp->GetNextSamples(buff, sizeof(buff)) / 2;

	int avg0 = 0;
	int avg1 = 0;

	if(l)
	{
		int i;
		for(i=0; i<l; i+=2)
		{
			avg0 += ABS(buff[i]);
			avg1 += ABS(buff[i+1]);
		}
		//avg0 /= l;
		//avg1 /= l;
		//cprintf("%d %d\n", l, avg0*2/l);
		lcd->SetLed(LedControl::BAR0, avg0 / (l*256));
		lcd->SetLed(LedControl::BAR1, avg1 / (l*256));

		for(i=0; i<l/4; i++)
			buff2[i] = (buff[i*2] + buff[i*2+1]) / 2;

		if(eq)
			eq->WriteSamples(buff2, l/4);
	}
}

	int hmargin = 8;
	int vmargin = 8;
	int sep = 8;



void layout()
{
	int fh = g_screen.GetFontHeight();

//	settings = *g_screen.GetSettings();

	int w = g_screen.Width() - hmargin*2;
	int h = g_screen.Height() - vmargin|2;

	lcd->Move(w - lcd->Width() + hmargin, fh+sep);
	pane->Move(hmargin, fh+sep);
	pane->Resize(w - lcd->Width() - sep, h - (fh+sep));
	pane->Clear(true);
	songInfo->Move(w - lcd->Width() + hmargin, lcd->Height() + (fh+sep) + sep);
	songInfo->Resize(lcd->Width(), fh * 9);
	songInfo->Clear(true);
	songInfo->Render();

	int y = lcd->Height() + (fh+sep) + sep + sep + fh * 9;
	eq->Move(w - lcd->Width() + hmargin, y);
	eq->Clear(true);

}

bool PlaySong(IFileList *list, int index)
{
	int fsize = list->GetSize(index);
	const char *name = list->GetName(index);
	IFileRef *fr = list->GetFileRef(index);

	bool ok = false;
	if(fr->GetFileName())
		ok = mp->PlaySong(fr->GetFileName());
	else
	{
		char *tmp = (char *)malloc(fsize);
		fr->Read(tmp, 0, fsize);
		ok = mp->PlaySong(name, tmp, fsize);
		free(tmp);
	}

	if(ok)
	{
		lcd->SetLed(LedControl::MAXSONG, mp->GetMaxTunes());
		lcd->SetLed(LedControl::SONGNO, mp->GetTune()+1);
		songInfo->SetName(mp->GetName());
		songInfo->SetArtist(mp->GetArtist());
		songInfo->SetCopyright(mp->GetCopyright());
		songInfo->SetLength(mp->GetLength());
		songInfo->SetFormat(mp->GetFormat());
	}
	else
	{
		cprintf("Could not play file\n");
	}

	delete fr;

	return ok;
}


static int bcount = 0;
extern "C" {

void busy(int x)
{
	if(x)
		bcount = (bcount+1) % 6;
	else
		bcount = 0;
	lcd->SetLed(LedControl::BUSY, bcount);
	lcd->Render();
}


}

int main(int argc, char **argv)
{
	settings.bevellight = 0xB2DBF8;
	settings.bevelmid = 0x5084A4;
	settings.beveldark = 0x1F596C;
	settings.bgcol = 0x000000;
	settings.textcol = 0xFFFFFF;

#ifdef GP2X

	settings.font0 = "";
	settings.font1 = "";
	settings.width = 320;
	settings.height = 240;
	char *led = "led.bmp";

#else

	settings.font0 = "Fixed.font";
	settings.font1 = "Proportional.font";
	settings.width = 640;
	settings.height = 570;
	char *led = "bigled.bmp";

#endif

	//g_screen.SetSettings(&settings);

	printf("Starting\n");
	g_screen.Init(&settings);
	printf("OK\n");

	pane = new ListPane(0, 0, 0, 0);
	songInfo = new SongInfoPane(0, 0, 0, 0);
	lcd = new LedControl(led, 0, 0);
	eq = new Equalizer(50, 50, lcd->Width(), lcd->Width()/3);

	printf("OK\n");

	layout();

	eq->Render();

	g_screen.Update();

	//g_screen.CreateConsole(w - lcd->Width() + hmargin, y, lcd->Width(), h-y); 

	printf("OK %d\n", songInfo->Height());

	//ListPane *pane = new ListPane(4,16,settings.width - 120, settings.height - 20);
	//TextDisplay *text = new TextDisplay(settings.width-104, 26, 104, 56);
	//LedControl *lcd = new LedControl("led5.bmp", 212, 90);

	mp = new MusicPlayer();
	Input *input = new Input();


	lcd->Render();
	//text->PutText(1,1,"B = Enter/Play");
	//text->PutText(1,2,"X = Parent dir");
	//text->PutText(1,3,"Y/A = Page Up/Down");
	//text->PutText(1,4,"Stick L/R = Subtune");
	//text->PutText(1,5,"Start = Quit");
	//text->Render();
	//songInfo->Render();

	char curdir[256];
	getcwd(curdir, sizeof(curdir));

	FileList *file_list = new FileList(curdir, &plugin_filter_cb);
	PlayList *play_list = new PlayList(&plugin_filter_cb);
	ZipFileList *zip_list = NULL;
//	FileList *play_list = NULL;
//	FileList *zip_list = NULL;

	pane->SetList(file_list);
	pane->SetTitle(file_list->GetCurDir());
	pane->SetTitleColor(0xE03050);
	pane->Render();

	int selected = -1;
	int repeat = 0;
	int shuffle = 0;
	int parent_selected = 0;
	IFileList *parent_file_list = NULL;
	int last_sec = -1;

	int paused = 0;
	int playing = 0;


	int oldplaying = -1;

	lcd->SetLed(LedControl::SONGNO, 1);
	lcd->SetLed(LedControl::MAXSONG, 1);
	lcd->SetLed(LedControl::VOLUME, mp->GetVolume());

	int ucount = 0;

	//int playlist_index = -1;

	int go_to = 0;
	int add_index = -1;

	while(1)
	{
		IFileList *current_list = pane->GetList();

		while(selected == -1)
		{
			SDL_Delay(50);
			pane->Render();
			songInfo->Render();
			lcd->Render();

			if(playing && !mp->Playing())
			{
				playing = 0;
				if(play_list->Size())
				{
					int i = play_list->Current();
					if(i >= 0 || repeat || go_to)
					{
						if(go_to)
						{
							play_list->SetCurrent(i+go_to);
							if(play_list->Current() == -1)
							{
								if(go_to > 0)
									play_list->SetCurrent(0);
								else
									play_list->SetCurrent(play_list->Size()-1);
							}
						}
						else
						{
							play_list->SetCurrent(i+1);
							if(repeat && play_list->Current() == -1)
								play_list->SetCurrent(0);

						}
						if(play_list->Current() != -1)
							playing = (int)PlaySong(play_list, play_list->Current());
					}
				}

			}
			go_to = 0;

			//songInfo->SetFormat(mp->GetFormat());
			//printf("%s\n", mp->GetFormat());

#ifdef GP2X
			if(!mp->GetHeavy())
#endif
			{
				if(eq)
					eq->Render();
				render_eq();
			}

			if(paused)
			{
				if(((paused-1) % 15) == 0)
					lcd->SetLed(LedControl::PLAYING, (paused-1) % 30 );
				paused++;
			}

			if(playing != oldplaying)
			{
				lcd->SetLed(LedControl::PLAYING, playing );	
				oldplaying = playing;
			}



			int sec = mp->GetCSeconds() / 10;

			if(last_sec != sec)
			{
				lcd->SetLed(LedControl::LENGTH, sec);
				last_sec = sec;
			}

			int key = input->GetKey(40);
			if(key)
			{
				switch(key)
				{
				case Input::PGUP:
					pane->DoCommand(ListPane::PGUP);
					break;
				case Input::PGDOWN:
					pane->DoCommand(ListPane::PGDN);
					break;
				case Input::UP:
					pane->DoCommand(ListPane::UP);
					break;
				case Input::DOWN:
					pane->DoCommand(ListPane::DOWN);
					break;
				case Input::NEXT:
					playing = 1;
					mp->Stop();
					go_to = 1;
					break;
				case Input::PREV:
					playing = 1;
					mp->Stop();
					go_to = -1;
					break;
				case Input::FF:
					mp->SetTune(mp->GetTune() + 1);
					lcd->SetLed(LedControl::SONGNO, mp->GetTune()+1);
					break;
				case Input::REW:
					mp->SetTune(mp->GetTune() - 1);
					lcd->SetLed(LedControl::SONGNO, mp->GetTune()+1);
					break;
				case Input::ENTER:
					selected = current_list->Marked();
					break;
				case Input::BACK:
					if(pane->GetList() != play_list)
						selected = -2;
					else
					{
						play_list->RemoveFile(current_list->Marked());
						pane->SetList(play_list);

					}
					break;
				case Input::VOLUP:
					mp->SetVolume(mp->GetVolume() + 1);
					lcd->SetLed(LedControl::VOLUME, mp->GetVolume());
					break;
				case Input::VOLDOWN:
					mp->SetVolume(mp->GetVolume() - 1);
					lcd->SetLed(LedControl::VOLUME, mp->GetVolume());
					break;
				case Input::QUIT:
					SDL_CloseAudio();
					SDL_Quit();
					return 0;
				case Input::SHUFFLE:
					shuffle ^= 1;
					lcd->SetLed(LedControl::SHUFFLE, shuffle);
					break;
				case Input::REPEAT:
					repeat ^= 1;
					lcd->SetLed(LedControl::REPEAT, repeat);
					break;
				case Input::PAUSE:
					paused = mp->Pause();
					break;
				case Input::BLENDUP:
#ifdef GP2X
					if(mp->GetBlend())
						mp->SetBlend(0);
					else
						mp->SetBlend(50);
#else
					mp->SetBlend(mp->GetBlend() + 25);
#endif
					lcd->SetLed(LedControl::EQ0, mp->GetBlend() / 50);
					lcd->SetLed(LedControl::EQ1, 5 - (mp->GetBlend() / 50));

					break;
				case Input::BLENDDOWN:
					mp->SetBlend(mp->GetBlend() - 25);
					lcd->SetLed(LedControl::EQ0, mp->GetBlend() / 50);
					lcd->SetLed(LedControl::EQ1, 5 - (mp->GetBlend() / 50));
				   	break;
				case Input::PLISTADD:
					if(pane->GetList() != play_list)
					{
						if(add_index != current_list->Marked())
						{
							add_index = current_list->Marked();
							const char *name = current_list->GetName(add_index);
							int size = current_list->GetSize(add_index);
							IFileRef *fr = current_list->GetFileRef(add_index);
							if(mp->CanHandle(name))
							{
								play_list->AddFile(name, size, fr);
								pane->DoCommand(ListPane::DOWN);
							}
						}
					}
					else
					{
						play_list->MoveFile(play_list->Marked(), play_list->Current() + 1);
						pane->SetList(play_list);
					}
					break;
				case Input::SCREENSHOT:
					g_screen.ScreenShot("screenshot.bmp");
					break;
				case Input::SELECT:
					if(pane->GetList() != play_list)
					{
						current_list = play_list;
						pane->SetTitle("PlayList");
						pane->SetTitleColor(0x90A040);
					}
					else
					{
						if(zip_list)
							current_list = zip_list;
						else
							current_list = file_list;
						pane->SetTitle(file_list->GetCurDir());
						pane->SetTitleColor(0xE03068);
					}
					pane->SetList(current_list);
					break;
				}
			}
		}

		add_index = -1;

		if(selected < 0)
		{
			int rc = current_list->Enter(-1);
			if(rc < 0)
			{
				delete current_list;
				current_list = parent_file_list;
				rc = parent_selected;
				parent_selected = 0;
				parent_file_list = NULL;
				zip_list = NULL;
			}
			pane->SetList(current_list);
			pane->SetTitle(file_list->GetCurDir());

			if(rc >= 0)
				current_list->SetMarked(rc);
			pane->DoCommand(ListPane::NOTHING);
		}
		else
		{
			int fsize = current_list->GetSize(selected);
			const char *name = current_list->GetName(selected);


			const char *ext = strrchr(name, '.');
			if(!ext)
				ext = "";

			if(fsize == -1)
			{
				current_list->Enter(selected);
				pane->SetList(current_list);
				pane->SetTitle(file_list->GetCurDir());
				selected = -1;
				current_list->SetMarked(0);
				current_list->SetStart(0);
				pane->DoCommand(ListPane::NOTHING);
			}
			else
			{
				
				if(stricmp(ext, ".zip") == 0)
				{
					IFileRef *fr = current_list->GetFileRef(selected);
					parent_file_list = current_list;
					parent_selected = selected;
					current_list = zip_list = new ZipFileList((char *)fr->GetFileName());
					pane->SetList(current_list);
					pane->SetTitle(file_list->GetCurDir());
					selected = -1;
					delete fr;
				}
				else
				{
					playing = (int)PlaySong(current_list, selected);//fr, name, fsize);
					if(current_list == play_list)
					{
						play_list->SetCurrent(selected);
						//playlist_index = selected+1;
						//if(playlist_index >= play_list->Size())
						//	playlist_index = 0;
					}
					else
					if(!repeat)
						play_list->SetCurrent(-1);
				}
			}


		}
		selected = -1;
	}

	return 0;
}
