/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef FILELIST_H
#define FILELIST_H

#include "Files.h"


class FileList : public IFileList
{
public:

	FileList(char *source, int (*fcb)(void *, const char *) = NULL, void *data = NULL);
	virtual ~FileList();

	//virtual int Read(int index, char *dest, int offset, int length);
	//virtual std::vector<FileRef> &GetNames();
	virtual int Enter(int index);

	virtual IFileRef *GetFileRef(int index);

	int Size() { return filerefs.size(); }

	/*virtual const char *GetFileName(int index)
	{
		return filerefs[index].path.c_str();
	}*/

	virtual const char *GetName(int index)
	{
		return filerefs[index].name.c_str();
	}

	virtual int GetSize(int index)
	{
		return filerefs[index].size;
	}

	virtual void SetFilterFunc(int (*fcb)(void *, const char *), void *data)
	{
		printf("Setting %p %p\n", filter_cb, &filter_cb);
		filter_cb = fcb;
		cb_data = data;
	}

	char *GetCurDir() { return curdir; }

protected:

	class RealFileRef : public IFileRef
	{
	public:
		RealFileRef(FileData &d)
		{
			data = d;
		}

		void FromString(std::string id)
		{
			int i = id.find_first_of(',');
			if(i != std::string::npos)
			{
				data.name = id.substr(0, i-1);
				data.path = id.substr(i+1);
			}

		}

		std::string ToString()
		{
			std::string s;
			s.append(data.name);
			s.append(",");
			s.append(data.path);
			return s;
		}



		int Read(char *dest, int offset, int length)
		{
			int rc = -1;
			FILE *fp = fopen(data.path.c_str(), "rb");
			if(fp)
			{
				fseek(fp, offset, SEEK_SET);
				rc = fread(dest, 1, length, fp);
				fclose(fp);
			}
			return rc;
		}

		const char *GetFileName() { return data.path.c_str(); }

	protected:
		FileData data;
	};

	void AddDirectory(char *dirname, bool recursive = false);

	int (*filter_cb)(void *, const char *);
	void *cb_data;

	//std::vector<char*> fnames;
	std::vector<FileData> filerefs;

	bool dirty;
	char curdir[256];

	void Enter(char *dirname);

};

#endif
