/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "PlayList.h"

#include "zipint.h"

#include <vector>
#include <algorithm>

#ifdef _WIN32
#define STRDUP _strdup
#else
#define STRDUP strdup
#endif

using namespace std;

PlayList::PlayList(int (*fcb)(void *, const char *), void *data) :
	FileList(NULL, fcb, data)
{
	shuffle_current = current = -1;
	changed = 0;
	shuffled = false;
	shuffle_on = false;
}

PlayList::~PlayList()
{
}

void PlayList::Shuffle()
{
	int i;
	if(filerefs.size())
	{
		shuffle_list.resize(filerefs.size());
		for(i=0; i<filerefs.size(); i++)
			shuffle_list[i] = i;
		std::random_shuffle(&shuffle_list[0], &shuffle_list[i-1]);
		shuffled = true;
	}
}

void PlayList::AddFile(const char *name, int size, IFileRef *fr)
{
	PLData data;

	data.name = string(name);
	data.size = size;
	data.fr = fr;
	filerefs.push_back(data);
	changed++;
}

void PlayList::Prev(bool wrap)
{
	//if(shuffle_on)
	//	current = shuffle_current;

	if(current == -1)
		current = filerefs.size();
	current--;
	changed++; 
	if(current < 0 && wrap)
		current = filerefs.size()-1;

	if(shuffle_on)
		shuffle_current = (current < 0) ? -1 : shuffle_list[current];

}

void PlayList::Next(bool wrap)
{
	//if(shuffle_on)
	//	current = shuffle_current;
	//if(current == -1)
	//	return;
	current++;
	changed++; 
	if(current >= filerefs.size())
	{
		if(wrap)
			current = 0;
		else
			current = -1;
	}

	if(shuffle_on)
		shuffle_current = (current < 0) ? -1 : shuffle_list[current];
}



int PlayList::Current()
{ 
	if(shuffle_on)
		return shuffle_current;
	else
		return current;
}


void PlayList::SetCurrent(int n)
{
	changed++; 

	shuffle_current = current = (n < 0 || n  >= (int)filerefs.size()) ? -1 : n;
/*
	if(shuffle_on)
	{
		if(!shuffled)
			Shuffle();
		shuffle_current = current;//(n < 0 || n  >= (int)filerefs.size()) ? -1 : shuffle_list[n];
	}
*/
}


IFileRef *PlayList::GetFileRef(int index)
{
	//if(shuffle_on)
	//	index = shuffle_list[index];
	return new PLFileRef(filerefs[index].fr);
}


void PlayList::RemoveFile(int n)
{

	if(n >= 0 && n < filerefs.size())
	{
		if(n == current)
			current = -1;
		else
		if(n < current)
			current--;
		filerefs.erase(filerefs.begin() + n);
		if(marked >= filerefs.size())
			marked--;
		changed++;
	}

	

}

void PlayList::MoveFile(int from, int to)
{
	if(to == current)
		current++;

	filerefs.insert(filerefs.begin() + to, *(filerefs.begin() + from));

	if(to < current)
		current--;

	if(from > to)
		from++;
	RemoveFile(from);



	changed++;
}


void PlayList::Save(const char *name)
{
	FILE *fp = fopen(name, "wb");
	if(fp)
	{
		for(int i=0; i<filerefs.size(); i++)
		{
			fprintf(fp, "%s\n", filerefs[i].fr->ToString().c_str());

		}
		fclose(fp);
	}
}

void PlayList::Load(const char *name)
{
	FILE *fp = fopen(name, "rb");
	if(fp)
	{
		//AddFile(
		fclose(fp);
	}
}
