/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdlib.h>
#include <SDL.h>
#include "Input.h"
#include "SDLScreen.h"

#define GP2X_BUTTON_UP              (0)
#define GP2X_BUTTON_UPLEFT          (1)
#define GP2X_BUTTON_LEFT            (2)
#define GP2X_BUTTON_DOWNLEFT        (3)
#define GP2X_BUTTON_DOWN            (4)
#define GP2X_BUTTON_DOWNRIGHT       (5)
#define GP2X_BUTTON_RIGHT           (6)
#define GP2X_BUTTON_UPRIGHT         (7)
#define GP2X_BUTTON_START           (8)
#define GP2X_BUTTON_SELECT          (9)
#define GP2X_BUTTON_R               (10)
#define GP2X_BUTTON_L               (11)
#define GP2X_BUTTON_A               (12)
#define GP2X_BUTTON_B               (13)
#define GP2X_BUTTON_Y               (14)
#define GP2X_BUTTON_X               (15)
#define GP2X_BUTTON_VOLUP           (16)
#define GP2X_BUTTON_VOLDOWN         (17)
#define GP2X_BUTTON_CLICK           (18)

#define MOD_L 0x20
#define MOD_R 0x40

#ifdef GP2X
unsigned char joy_actions[0x80];
#else
unsigned char key_actions[SDLK_LAST];
#endif

#define KEYREP 0x80

Input::Input()
{
#ifdef GP2X
	joy_actions[GP2X_BUTTON_UPLEFT]    = UP   | KEYREP;
	joy_actions[GP2X_BUTTON_UP]        = UP   | KEYREP;
	joy_actions[GP2X_BUTTON_UPRIGHT]   = UP   | KEYREP;
	joy_actions[GP2X_BUTTON_DOWNLEFT]  = DOWN | KEYREP;
	joy_actions[GP2X_BUTTON_DOWN]      = DOWN | KEYREP;
	joy_actions[GP2X_BUTTON_DOWNRIGHT] = DOWN | KEYREP;
	joy_actions[GP2X_BUTTON_LEFT]      = REW;
	joy_actions[GP2X_BUTTON_RIGHT]     = FF;

	joy_actions[GP2X_BUTTON_A]       = NEXT;
    joy_actions[GP2X_BUTTON_B]       = ENTER;
	joy_actions[GP2X_BUTTON_X]       = BACK;
	joy_actions[GP2X_BUTTON_Y]       = PLISTADD | KEYREP;
    joy_actions[GP2X_BUTTON_START]   = PAUSE;
	joy_actions[GP2X_BUTTON_SELECT]  = SELECT;
	joy_actions[GP2X_BUTTON_VOLUP]   = VOLUP   | KEYREP;
	joy_actions[GP2X_BUTTON_VOLDOWN] = VOLDOWN | KEYREP;
	joy_actions[GP2X_BUTTON_L]       = SAVE;
	joy_actions[GP2X_BUTTON_CLICK]   = SCREENSHOT;

	joy_actions[MOD_R | GP2X_BUTTON_UPLEFT]    = PGUP   | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_UP]        = PGUP   | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_UPRIGHT]   = PGUP   | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_DOWNLEFT]  = PGDOWN | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_DOWN]      = PGDOWN | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_DOWNRIGHT] = PGDOWN | KEYREP;

	joy_actions[MOD_R | GP2X_BUTTON_A]       = PREV;
	joy_actions[MOD_R | GP2X_BUTTON_B]       = SHUFFLE;
	joy_actions[MOD_R | GP2X_BUTTON_X]       = DELETE | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_Y]       = REPEAT;
	joy_actions[MOD_R | GP2X_BUTTON_START]   = QUIT;
	joy_actions[MOD_R | GP2X_BUTTON_SELECT]  = BLANK;
	joy_actions[MOD_R | GP2X_BUTTON_VOLUP]   = BLENDUP | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_VOLDOWN] = BLENDDOWN | KEYREP;
	joy_actions[MOD_R | GP2X_BUTTON_L]       = HOLD;
	joy_actions[MOD_R | GP2X_BUTTON_CLICK]   = SPEED;
#else
	key_actions[SDLK_LEFT]      = REW;
	key_actions[SDLK_RIGHT]     = FF;
	key_actions[SDLK_UP]        = UP | KEYREP;
	key_actions[SDLK_DOWN]      = DOWN | KEYREP;
	key_actions[SDLK_F2]        = VOLUP | KEYREP;
	key_actions[SDLK_F1]        = VOLDOWN | KEYREP;
	key_actions[SDLK_RETURN]    = ENTER;
	key_actions[SDLK_BACKSPACE] = BACK;
	key_actions[SDLK_PAGEUP]    = PGUP | KEYREP;
	key_actions[SDLK_PAGEDOWN]  = PGDOWN | KEYREP;
	key_actions[SDLK_ESCAPE]    = QUIT;
	key_actions['r']            = REPEAT;
	key_actions['s']            = SHUFFLE;
	key_actions[' ']            = PAUSE;
	key_actions['p']            = PREV;
	key_actions['n']            = NEXT;
	key_actions[SDLK_F9]        = BLANK;
	key_actions[SDLK_F10]       = HOLD;
	key_actions['a']            = PLISTADD | KEYREP;
	key_actions[SDLK_TAB]       = SELECT;
	key_actions[SDLK_F4]        = BLENDUP | KEYREP;
	key_actions[SDLK_F3]        = BLENDDOWN | KEYREP;
	key_actions[SDLK_F11]       = SCREENSHOT;
#endif
}

unsigned char last_button = 0;
int repeat = 0;
int repeat_first = 160;
int repeat_rate = 40;
int mods = 0;

int Input::GetKey(int msec)
{
	SDL_Event evt;
	int ie = SDL_PollEvent(&evt);
	if(ie)
	{
		switch(evt.type)
		{
#ifdef GP2X
		case SDL_JOYBUTTONUP:
		    //if(evt.jbutton.button == GP2X_BUTTON_L)
			//    mods &= ~MOD_L;
			//else
			if(evt.jbutton.button == GP2X_BUTTON_R)
				mods &= ~MOD_R;
			last_button = 0;
			break;
		case SDL_JOYBUTTONDOWN:
			//if(evt.jbutton.button == GP2X_BUTTON_L)
			//    mods |= MOD_L;
			//else
			if(evt.jbutton.button == GP2X_BUTTON_R)
				mods |= MOD_R;
			else
			{
				last_button = joy_actions[evt.jbutton.button | mods];
				repeat = repeat_first;
			}
			return last_button & (~KEYREP);
			break;
#else
		case SDL_KEYDOWN:
			last_button = key_actions[evt.key.keysym.sym];
			repeat = repeat_first;
			return last_button & (~KEYREP);
			break;
		case SDL_VIDEORESIZE:
			g_screen.Resize(evt.resize.w, evt.resize.h);
			return RESIZE;
			break;
		case SDL_QUIT:
			exit(0);
#endif
		}
	}

	if(last_button & KEYREP)
	{
		repeat -= msec;
		if(repeat <= 0)
		{
			repeat += repeat_rate;
			return last_button & (~KEYREP);
		}
	}
	return NONE;
}
