/*  
    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>

#include "pokecubed/pokecubed.h"
#include "../plugin.h"
#include "../util.h"

static struct sound_plugin plugin;
static FILE *fp;
pokecubed *cubeplayer;

static int init_file(char *fname)
{
    cubeplayer = new pokecubed();
    cubeplayer->init();
    if (!cubeplayer->isFilePlayable(fname) || !cubeplayer->loadFile(fname))
    {
        fprintf(stderr,"pokecubeplugin:init_file - Couldn't load file");
        return -1;
    }
    
    plugin.name = "";
    plugin.subname = "";
    plugin.artist = "";
    plugin.copyright = "";
    plugin.length = cubeplayer->getLength();
    if (plugin.length <= 0)
        plugin.length = 150000;
    printf("samples  %d - length %d\n",cubeplayer->getNumberOfSamples(),cubeplayer->getLength());
    printf("using length %d\n",plugin.length);
    plugin.subtunes = 1;
    plugin.tune = 0;
    plugin.channels = cubeplayer->getNumberOfChannels();
    plugin.freq = cubeplayer->getFrequency();
    return 0;
}

static int close()
{
    delete cubeplayer;
    return 0;
}

static int fill_buffer(signed short *dest, int len)
{
    int samp_per_chn = cubeplayer->getNSamples(dest,len/(plugin.channels<<1));
    return samp_per_chn*plugin.channels<<1;
}

static int set_position(int seconds, int subtune)
{
    if (seconds <= 0)
        return 0;
    else
        return cubeplayer->seek((unsigned int)10*seconds);
}

static int can_handle(const char *name)
{
    return (is_ext(name, ".adx") || is_ext(name,".adp") 
         || is_ext(name, ".dsp") || is_ext(name,".gcm")
         || is_ext(name, ".afc") || is_ext(name,".ast")
         || is_ext(name, ".hps") || is_ext(name,".idsp")
         || is_ext(name, ".spt") || is_ext(name,".spd")
         || is_ext(name, ".mss") || is_ext(name,".mpdsp")
         || is_ext(name, ".ish") || is_ext(name,".ymf")
         || is_ext(name, ".rsd") || is_ext(name,".rsp"));
}

extern "C" {

#ifndef INIT_SOUND_PLUGIN
#define INIT_SOUND_PLUGIN pokecubed_init_sound_plugin
#endif

struct sound_plugin *INIT_SOUND_PLUGIN()
{
    memset(&plugin, 0, sizeof(plugin));
    plugin.plugname = "pokecubed";
    //plugin.init_data = init_data;
    plugin.init_file = init_file;
    //plugin.request_format = NULL;
    plugin.set_position = set_position;
    plugin.fill_buffer = fill_buffer;
    plugin.can_handle = can_handle;
    plugin.close = close;
    plugin.clockfreq = 200;
    plugin.format = "GameCube";
    return &plugin;
}

}

